#### 自旋锁

​	自旋锁是一种保护短持续时间临界区的概念锁。要访问临界区，就必须获得其相关的自旋锁。它会反复尝试获取自旋锁，直到成功为止。当临界区完成后，则会释放自旋锁，从而允许另一个进程获取该自旋锁。



#### 实现

```rust
pub struct SpinMutex<T: ?Sized> {
    pub(crate) lock: AtomicBool,
    pub(crate) cpu: UnsafeCell<isize>,
    data: UnsafeCell<T>,
}
```

SpinMutex结构体中保存了lock，cpu，data3个字段。同时在在Drop时释放锁。

lock：锁字段

cpu：当前获得锁的hartid

data：自旋锁保存的数据

```rust
impl<'a, T: ?Sized> Drop for SpinMutexGuard<'a, T> {
    fn drop(&mut self) {
        let cpu = *self.cpu;
        let current = current_hartid();
        if cpu != current as isize {
            panic!("release wrong cpu {} {}",cpu, current);
        }
        *self.cpu = -1;
        self.lock.store(false, Ordering::Release);
    }
}
```



#### Hartid

LotusOS中通过读取tp寄存器来识别hartid。

```rust
pub fn current_hartid() -> usize {
    let mut tp: usize;
    unsafe { asm!("mv {}, tp", out(reg) tp) };
    tp
}
```

LotusOS会在开始时将tp设置为hartid

```rust
pub fn rust_main(hartid: usize) -> ! {
    unsafe { asm!("mv tp, {}", in(reg)hartid) }
    ...
    ...
}
```

编译器默认不是使用tp寄存器，但是用户态可能会修改tp寄存器，因此我们需要再特权级转换时确保tp寄存器被保存，避免在S态读取到被U态修改的tp寄存器。

```assembly
trap.S

__alltrap:
	...
	ld tp, 37*8(sp)
	...
__restore:
	...
	sd tp, 37*8(sp)
	...
```

