use crate::drivers::{PlicDevice, UartDevice};

const PLIC_PROIRITY: usize = 0x0C00_0000;
const PLIC_THRESHOLD: usize = 0x0C20_1000;
const PLIC_ENABLE: usize = 0x0C00_2080;
const PLIC_CLAIM: usize = 0x0c20_1004;

pub struct Plic {}

impl Plic {
    pub fn new() -> Self {
        Self {}
    }
}

impl PlicDevice for Plic {
    fn enable(&self, source: usize, _hartid: usize) {
        unsafe {
            let enables = PLIC_ENABLE as *mut u32;
            enables.write_volatile(enables.read_volatile() | 1 << source);
        }
    }

    fn current(&self, hartid: usize) -> usize {
        unsafe {
            let claim_reg = PLIC_CLAIM as *const u32;
            claim_reg.read_volatile() as usize
        }
    }

    fn complete(&self, irq: usize, hartid: usize) {
        let complete_reg = PLIC_CLAIM as *mut u32;
        unsafe {
            complete_reg.write_volatile(irq as u32);
        }
    }

    fn set_priority(&self, value: u32, pin: usize) {
        unsafe {
            let ptr = PLIC_PROIRITY as *mut u32;
            ptr.add(pin).write_volatile(value);
        }
    }

    fn set_thershold(&self, value: u32, hartid: usize) {
        unsafe {
            let ptr = PLIC_THRESHOLD as *mut u32;
            ptr.write_volatile((value & 0b111) as u32);
        }
    }
}
