use alloc::collections::VecDeque;

use crate::{drivers::UartDevice, irq::wait_for_irq_and_run_next, sync::UPSafeCell};

pub struct UartHs {
    #[allow(unused)]
    buffer: UPSafeCell<VecDeque<u8>>,
}

impl UartHs {
    pub fn new() -> Self {
        Self {
            buffer: unsafe { UPSafeCell::new(VecDeque::new()) },
        }
    }
}

impl UartDevice for UartHs {
    fn getchar(&self) -> Option<u8> {
        let mut buffer = self.buffer.exclusive_access();
        let len = buffer.len();
        drop(buffer);
        if len == 0 {
            wait_for_irq_and_run_next(33);
        }
        let mut buffer = self.buffer.exclusive_access();
        assert!(buffer.len() > 0);
        buffer.pop_front()
    }

    fn putchar(&self, ch: u8) {
        unsafe {
            let ptr = k210_pac::UARTHS::ptr();
            while (*ptr).txdata.read().full().bit() {
                continue;
            }
            (*ptr).txdata.write(|w| w.data().bits(ch))
        }
    }

    fn handler_interrupt(&self) {
        unsafe {
            let ptr = k210_pac::UARTHS::ptr();
            let recv = (*ptr).rxdata.read();
            match recv.empty().bits() {
                true => panic!("Not char"),
                false => {
                    let ch = recv.data().bits();
                    self.buffer.exclusive_access().push_back(ch);
                }
            }
        }
    }
}
