use super::File;
use crate::drivers::UART_DEVICE;
use crate::irq::wait_for_irq_and_run_next;
use crate::mm::UserBuffer;
use crate::sbi::console_getchar;
use alloc::string::String;
use k210_pac::Interrupt;
use riscv::register::sstatus;
pub struct Stdin;

pub struct Stdout;

impl File for Stdin {
    fn readable(&self) -> bool {
        true
    }
    fn writable(&self) -> bool {
        false
    }
    fn read(&self, mut user_buf: UserBuffer) -> usize {
        for i in user_buf {
            let mut ch = UART_DEVICE.getchar().unwrap();
            // let ch = console_getchar() as u8;
            unsafe { *i = ch }
        }
        1
    }

    fn name(&self) -> String {
        String::from("Stdin")
    }
}

impl File for Stdout {
    fn readable(&self) -> bool {
        false
    }
    fn writable(&self) -> bool {
        true
    }

    fn write(&self, user_buf: UserBuffer) -> usize {
        for buffer in user_buf.buffers.iter() {
            print!("{}", core::str::from_utf8(*buffer).unwrap());
        }
        user_buf.len()
    }

    fn name(&self) -> String {
        String::from("Stdout")
    }
}
