#![no_std]
#![no_main]
#![allow(unused)]
#![allow(non_snake_case)]
#![allow(non_camel_case_types)]
#![feature(map_try_insert)]
#![feature(array_methods)]
#![feature(fn_traits)]
#![feature(const_btree_new)]
#![feature(panic_info_message)]
#![feature(alloc_error_handler)]

use crate::drivers::BLOCK_DEVICE;
use core::{
    arch::{asm, global_asm},
    sync::atomic::{fence, AtomicBool, Ordering},
};
use riscv::{register::sstatus::{self, FS}, asm::wfi};

extern crate alloc;
#[macro_use]
extern crate bitflags;
#[macro_use]
mod console;
#[macro_use]
mod config;
mod drivers;
mod fatfs;
mod fs;
mod irq;
mod lang_items;
mod mm;
mod sbi;
mod sync;
mod syscall;
mod task;
mod timer;
mod trap;
global_asm!(include_str!("entry.asm"));

fn clear_bss() {
    extern "C" {
        fn sbss();
        fn ebss();
    }
    unsafe {
        core::slice::from_raw_parts_mut(sbss as usize as *mut u8, ebss as usize - sbss as usize)
            .fill(0);
    }
}

static mut LOCK: bool = true;

#[no_mangle]
pub unsafe fn rust_main(hartid: usize) -> ! {
    if hartid == 0 {
        clear_bss();
        mm::init();
        mm::remap_test();
        console::logger_init();
        println!("[kernel] Lotus core {} boot", hartid);
        println!("{}", include_str!("banner"));
        sstatus::set_fs(FS::Initial);
        trap::init();
        irq::irq_init(hartid);
        fatfs::fs_init();
        task::add_initproc();
        trap::enable_timer_interrupt();
        timer::set_next_trigger();
        #[cfg(feature = "board_k210")]
        {
            // BLOCK_DEVICE.change_mode();
            sbi::send_ipi(1);
        }
        // LOCK = false;
    } else {
        unsafe{ wfi() }
        let lock = &LOCK as *const bool;
        while lock.read_volatile() {}
        sstatus::set_fs(FS::Initial);
        mm::activate();
        trap::init();
        // irq::irq_init(hartid);
        trap::enable_timer_interrupt();
        timer::set_next_trigger();
        println!("[kernel] Lotus core {} boot", hartid);
    }
    task::run_tasks(hartid);
    unreachable!("Unreachable in rust_main!");
}
