mod address;
mod frame_allocator;
mod heap_allocator;
mod memory_set;
mod page_table;

use address::VPNRange;
pub use address::{PhysAddr, PhysPageNum, StepByOne, VirtAddr, VirtPageNum};
pub use frame_allocator::{frame_alloc, frame_dealloc, FrameTracker, FRAME_ALLOCATOR};
use log::info;
pub use memory_set::remap_test;
pub use memory_set::{kernel_token, MapPermission, MapType, MemorySet, KERNEL_SPACE};
use page_table::PTEFlags;
pub use page_table::{
    translated_byte_buffer, translated_bytes, translated_copy, translated_copy_ref,
    translated_copy_refmut, translated_ref, translated_refmut, translated_str, PageTable,
    PageTableEntry, UserBuffer, UserBufferIterator,
};
use riscv::register::scause::Trap;

use crate::task::{self, current_process};
use crate::trap::TrapContext;

pub fn init() {
    heap_allocator::init_heap();
    task::init_hart();
    frame_allocator::init_frame_allocator();
    let kernel_space = KERNEL_SPACE.lock().unwrap();
    kernel_space.activate();
}
pub fn activate() {
    let kernel_space = KERNEL_SPACE.lock().unwrap();
    kernel_space.activate();
}

pub fn do_page_fault(cx: &mut TrapContext, stval: usize, trap: Trap) -> Result<(), ()>{
    let process = current_process();
    let mut inner = process.inner_lock_access().unwrap();
    let mut memory_set = inner.memory_set.lock().unwrap();
    MemorySet::handler_pte_fault(&mut memory_set, stval, trap)
}
