const SYSCALL_GET_CWD: usize = 17;
const SYSCALL_DUP: usize = 23;
const SYSCALL_DUP3: usize = 24;
const SYSCALL_FCNTL: usize = 25;
const SYSCALL_IOCTL: usize = 29;
const SYSCALL_MKDIR: usize = 34;
const SYSCALL_UNLINK: usize = 35;
const SYSCALL_UMOUNT: usize = 39;
const SYSCALL_MOUNT: usize = 40;
const SYSCALL_STATVFS: usize = 43;
const SYSCALL_FACCESSAT: usize = 48;
const SYSCALL_CHDIR: usize = 49;
const SYSCALL_OPEN: usize = 56;
const SYSCALL_CLOSE: usize = 57;
const SYSCALL_PIPE: usize = 59;
const SYSCALL_GETDENTS: usize = 61;
const SYSCALL_LSEEK: usize = 62;
const SYSCALL_READ: usize = 63;
const SYSCALL_WRITE: usize = 64;
const SYSCALL_READV: usize = 65;
const SYSCALL_WRITEV: usize = 66;
const SYSCALL_PREAD: usize = 67;
const SYSCALL_PWRITE: usize = 68;
const SYSCALL_ADJTIMEX: usize = 71;
const SYSCALL_FSTATAT: usize = 79;
const SYSCALL_FSTAT: usize = 80;
const SYSCALL_UTIMENSAT: usize = 88;
const SYSCALL_EXIT: usize = 93;
const SYSCALL_EXIT_GROUP: usize = 94;
const SYSCALL_SET_TID_ADDR: usize = 96;
const SYSCALL_FUTEX: usize = 98;
const SYSCALL_SET_ROBUST_LIST: usize = 99;
const SYSCALL_GET_ROUBST_LIST: usize = 100;
const SYSCALL_SLEEP: usize = 101;
const SYSCALL_CLK_GETTIME: usize = 113;
const SYSCALL_LOG: usize = 116;
const SYSCALL_YIELD: usize = 124;
const SYSCALL_TKILL: usize = 130;
const SYSCALL_SIGACTION: usize = 134;
const SYSCALL_SIGPROCMASK: usize = 135;
const SYSCALL_SIGTIMEDWAIT: usize = 137;
const SYSCALL_SIGRETURN: usize = 139;
const SYSCALL_GETRESUID: usize = 144;
const SYSCALL_TIMES: usize = 153;
const SYSCALL_UNAME: usize = 160;
const SYSCALL_GET_TIME: usize = 169;
const SYSCALL_GETPID: usize = 172;
const SYSCALL_GETPPID: usize = 173;
const SYSCALL_GETUID: usize = 174;
const SYSCALL_GETEUID: usize = 175;
const SYSCALL_GETGID: usize = 176;
const SYSCALL_GETEGID: usize = 177;
const SYSCALL_GETTID: usize = 178;
const SYSCALL_INFO: usize = 179;
const SYSCALL_SOCKET: usize = 198;
const SYSCALL_BIND: usize = 200;
const SYSCALL_GETSOCKNAME: usize = 204;
const SYSCALL_SENDTO: usize = 206;
const SYSCALL_RECVFROM: usize = 207;
const SYSCALL_SETSOCKOPT: usize = 208;
const SYSCALL_BRK: usize = 214;
const SYSCALL_MUNMAP: usize = 215;
const SYSCALL_CLONE: usize = 220;
const SYSCALL_EXEC: usize = 221;
const SYSCALL_MMAP: usize = 222;
const SYSCALL_MPROTECT: usize = 226;
const SYSCALL_WAITPID: usize = 260;
const SYSCALL_PRLIMIT: usize = 261;
const SYSCALL_MEMBARRER: usize = 283;
const SYSCALL_THREAD_CREATE: usize = 1000;
// const SYSCALL_GETTID: usize = 1001;
const SYSCALL_WAITTID: usize = 1002;
const SYSCALL_MUTEX_CREATE: usize = 1010;
const SYSCALL_MUTEX_LOCK: usize = 1011;
const SYSCALL_MUTEX_UNLOCK: usize = 1012;
const SYSCALL_SEMAPHORE_CREATE: usize = 1020;
const SYSCALL_SEMAPHORE_UP: usize = 1021;
const SYSCALL_SEMAPHORE_DOWN: usize = 1022;

mod fs;
mod net;
mod sync;
mod osinfo;
mod process;
// mod thread;
use crate::{
    fs::{Dirent, Kstat},
    task::Tms,
    timer::TimeVal,
};
use fs::*;
pub use fs::{MmapFlag, MmapProt};
use k210_pac::dmac::id;
use osinfo::*;
use process::*;
use riscv::register::sepc;
pub use sync::WAIT_TASK_LIST;
use sync::*;
use net::*;
// use thread::*;

pub fn syscall(syscall_id: usize, args: [usize; 6]) -> isize {
    match syscall_id {
        SYSCALL_GET_CWD => sys_getcwd(args[0] as *const u8, args[1]),
        SYSCALL_DUP => sys_dup(args[0]),
        SYSCALL_DUP3 => sys_dup3(args[0], args[1]),
        SYSCALL_FCNTL => sys_fcntl(args[0] as isize, args[1] as isize),
        SYSCALL_IOCTL => sys_ioctl(args[0], args[1]),
        SYSCALL_MKDIR => sys_mkdir(args[0] as isize, args[1] as *const u8, args[2]),
        SYSCALL_UMOUNT => sys_umount(args[0] as *const u8),
        SYSCALL_MOUNT => sys_mount(args[0] as *const u8, args[1] as *const u8),
        SYSCALL_STATVFS => sys_statvfs(args[0] as isize, args[1]),
        SYSCALL_FACCESSAT => 0,
        SYSCALL_UNLINK => sys_unlink(args[0] as isize, args[1] as *const u8, args[2]),
        SYSCALL_CHDIR => sys_chdir(args[0] as *const u8),
        SYSCALL_OPEN => sys_open(args[0] as isize, args[1] as *const u8, args[2] as i32),
        SYSCALL_CLOSE => sys_close(args[0]),
        SYSCALL_PIPE => sys_pipe(args[0] as *mut i32),
        SYSCALL_LSEEK => sys_lseek(args[0] as isize, args[1] as isize, args[2] as isize),
        SYSCALL_GETDENTS => sys_getdents(args[0] as isize, args[1] as *mut Dirent, args[2]),
        SYSCALL_READ => sys_read(args[0], args[1] as *const u8, args[2], -1),
        SYSCALL_WRITE => sys_write(args[0], args[1] as *const u8, args[2]),
        SYSCALL_READV => sys_readv(args[0], args[1], args[2]),
        SYSCALL_WRITEV => sys_writev(args[0], args[1], args[2]),
        SYSCALL_PREAD => sys_read(args[0], args[1] as *const u8, args[2], args[3] as isize),
        SYSCALL_PWRITE => sys_write(args[0], args[1] as *const u8, args[2]),
        SYSCALL_ADJTIMEX => sys_adjtimex(args[0] as *const u8),
        SYSCALL_FSTATAT => sys_fstatat(
            args[0] as isize,
            args[1] as *const u8,
            args[2] as *const u8,
            args[3] as isize,
        ),
        SYSCALL_FSTAT => sys_fstat(args[0] as isize, args[1] as *const u8),
        SYSCALL_UTIMENSAT => sys_utimensat(
            args[0] as isize,
            args[1] as *const u8,
            args[2] as *const u8,
            args[3] as isize,
        ),
        SYSCALL_EXIT => sys_exit(args[0] as i32),
        SYSCALL_EXIT_GROUP => sys_exitgroup(args[0] as i32),
        SYSCALL_SET_TID_ADDR => sys_set_tid_addr(args[0]),
        SYSCALL_FUTEX => sys_futex(
            args[0],
            args[1],
            args[2] as i32,
            args[3],
            args[4],
            args[5] as i32,
        ),
        SYSCALL_SET_ROBUST_LIST => sys_set_robust_list(args[0], args[1]),
        SYSCALL_GET_ROUBST_LIST => sys_get_robust_list(args[0], args[1], args[2]),
        SYSCALL_SLEEP => sys_sleep(args[0] as *const TimeVal),
        SYSCALL_CLK_GETTIME => sys_clk_gettime(args[0], args[1] as *const u8),
        SYSCALL_LOG => sys_log(args[0] as isize, args[1] as *const u8, args[2] as isize),
        SYSCALL_YIELD => sys_yield(),
        SYSCALL_TKILL => sys_tkill(args[0] as isize, args[1] as isize),
        SYSCALL_SIGACTION => sys_sigaction(args[0] as isize, args[1], args[2]),
        SYSCALL_SIGPROCMASK => sys_sigprocmask(args[0] as isize, args[1], args[2]),
        SYSCALL_SIGTIMEDWAIT => sys_sigtimedwait(),
        SYSCALL_SIGRETURN => sys_sigreturn(),
        SYSCALL_GETRESUID => sys_getresuid(),
        SYSCALL_TIMES => sys_times(args[0] as *const u8),
        SYSCALL_UNAME => sys_uname(args[0] as *const u8),
        SYSCALL_GET_TIME => sys_get_time(args[0] as *mut TimeVal),
        SYSCALL_GETPID => sys_getpid(),
        SYSCALL_GETPPID => sys_getppid(),
        SYSCALL_GETUID => sys_getuid(),
        SYSCALL_GETEUID => sys_geteuid(),
        SYSCALL_GETGID => sys_getgid(),
        SYSCALL_GETEGID => sys_getegid(),
        SYSCALL_GETTID => sys_get_tid(),
        SYSCALL_INFO => sys_sysinfo(args[0] as *const u8),
        SYSCALL_SOCKET => sys_socket(args[0] as isize, args[1] as isize, args[2] as isize),
        SYSCALL_BIND => sys_bind(args[0] as isize, args[1], args[2] as isize),
        SYSCALL_GETSOCKNAME => sys_getsockname(args[0] as isize, args[1], args[2] as isize),
        SYSCALL_SENDTO => sys_sendto(args[0] as isize, args[1], args[2] as isize, args[3] as isize, args[4], args[5] as isize),
        SYSCALL_RECVFROM => sys_recvfrom(args[0] as isize, args[1], args[2] as isize, args[3] as isize, args[4], args[5] as isize),
        SYSCALL_SETSOCKOPT => sys_setsockopt(args[0] as isize, args[1] as isize, args[2] as isize, args[3], args[4] as isize),
        SYSCALL_BRK => sys_brk(args[0]),
        SYSCALL_MUNMAP => sys_munmap(args[0], args[1]),
        SYSCALL_CLONE => sys_fork(args[0], args[1], args[2], args[3], args[4]),
        SYSCALL_EXEC => sys_exec(args[0] as *const u8, args[1] as *const usize),
        SYSCALL_MMAP => sys_mmap(
            args[0],
            args[1],
            args[2],
            args[3],
            args[4] as isize,
            args[5],
        ),
        SYSCALL_MPROTECT => sys_mprotect(args[0], args[1], args[2]),
        SYSCALL_WAITPID => sys_waitpid(args[0] as isize, args[1] as *mut i32, args[2] as isize),
        SYSCALL_PRLIMIT => sys_prlimit(args[0], args[1] as isize, args[2], args[3]),
        SYSCALL_MEMBARRER => sys_membarrier(args[0] as isize, args[1], args[2] as isize),
        // SYSCALL_THREAD_CREATE => sys_thread_create(args[0], args[1]),
        // SYSCALL_WAITTID => sys_waittid(args[0]) as isize,
        SYSCALL_MUTEX_CREATE => sys_mutex_create(args[0] == 1),
        SYSCALL_MUTEX_LOCK => sys_mutex_lock(args[0]),
        SYSCALL_MUTEX_UNLOCK => sys_mutex_unlock(args[0]),
        SYSCALL_SEMAPHORE_CREATE => sys_semaphore_creare(args[0]),
        SYSCALL_SEMAPHORE_UP => sys_semaphore_up(args[0]),
        SYSCALL_SEMAPHORE_DOWN => sys_semaphore_down(args[0]),
        0 => {println!("Catch Syscall 0"); 0},
        _ => panic!(
            "Unsupported syscall_id: {} sepc: {:#x}",
            syscall_id,
            sepc::read()
        ),
    }
}
