use alloc::sync::Arc;

use crate::{task::{current_process, current_user_token}, fs::{Socket, SockAddr}, mm::{translated_copy_refmut, translated_copy, UserBuffer, translated_byte_buffer}};

use super::process;



pub fn sys_socket(domaic: isize, r#type: isize, protocal: isize) -> isize {
    let process = current_process();
    let process_inner = process.inner_lock_access().unwrap();
    let mut fd_table = process_inner.fd_table.lock().unwrap();
    let fd = fd_table.alloc_fd();
    let socket = Socket::new();
    fd_table.tables.insert(fd, Some(Arc::new(socket)));
    fd as isize
}
pub fn sys_bind(sockfd: isize, addr: usize, addrlen: isize) -> isize {
    let process = current_process();
    let process_inner = process.inner_lock_access().unwrap();
    let token = process_inner.get_user_token();
    let mut fd_table = process_inner.fd_table.lock().unwrap();
    if let Some(Some(fd)) = fd_table.tables.get_mut(&(sockfd as usize)) {
        let sockaddr: SockAddr = translated_copy(token, addr, addrlen as usize);
        println!("{:?}",sockaddr);
        fd.bind(sockaddr, addrlen);
    }
    0
}
pub fn sys_getsockname(sockfd: isize, addr: usize, addrlen: isize) -> isize {
    0
}
pub fn sys_setsockopt(sockfd: isize, level: isize, optname: isize, optval: usize, optlen: isize) -> isize {
    0
}
pub fn sys_sendto(sockfd: isize, buf: usize, len: isize, flags: isize, addr: usize, addrlen: isize) -> isize {
    let process = current_process();
    let process_inner = process.inner_lock_access().unwrap();
    let token = process_inner.get_user_token();
    let mut fd_table = process_inner.fd_table.lock().unwrap();
    let sockaddr: SockAddr = translated_copy(token, addr, addrlen as usize);
    println!("SockAddr {:?}",sockaddr);
    if let Some(Some(fd)) = fd_table.tables.get_mut(&(sockfd as usize)) {
        let user_buf = UserBuffer::new(translated_byte_buffer(token, buf as *const u8, len as usize));
        println!("Send {} Len {:?}",sockfd, user_buf.len());
        fd.write(user_buf) as isize
    } else {
        0
    }
}
pub fn sys_recvfrom(sockfd: isize, buf: usize, len: isize, flags: isize, addr: usize, addrlen: isize) -> isize {
    let process = current_process();
    let process_inner = process.inner_lock_access().unwrap();
    let token = process_inner.get_user_token();
    let mut fd_table = process_inner.fd_table.lock().unwrap();
    if let Some(Some(fd)) = fd_table.tables.get_mut(&(sockfd as usize)) {
        let user_buf = UserBuffer::new(translated_byte_buffer(token, buf as *const u8, len as usize));
        println!("RecvFrom {} Len {:?}",sockfd, user_buf.len());
        fd.read(user_buf) as isize
    } else {
        0
    }
}