use core::fmt::Debug;

use crate::sync::{SpinMutex, UPSafeCell};

use super::TaskControlBlock;
use alloc::collections::VecDeque;
use alloc::sync::Arc;
use lazy_static::*;
pub struct TaskManager {
    pub ready_queue: VecDeque<Arc<TaskControlBlock>>,
}

/// A simple FIFO scheduler.
impl TaskManager {
    pub fn new() -> Self {
        Self {
            ready_queue: VecDeque::new(),
        }
    }
    pub fn add(&mut self, task: Arc<TaskControlBlock>) {
        self.ready_queue.push_back(task);
    }

    pub fn fetch(&mut self) -> Option<Arc<TaskControlBlock>> {
        self.ready_queue.pop_front()
    }
}

lazy_static! {
    pub static ref TASK_MANAGER: SpinMutex<TaskManager> = SpinMutex::new(TaskManager::new());
}

pub fn add_task(task: Arc<TaskControlBlock>) {
    // TASK_MANAGER.lock().unwrap().add(task);
    match TASK_MANAGER.lock() {
        Ok(mut manager) => manager.add(task),
        Err(e) => panic!("add {:?}", e),
    }
}

pub fn fetch_task() -> Option<Arc<TaskControlBlock>> {
    TASK_MANAGER.lock().unwrap().fetch()
}

impl Debug for TaskManager {
    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
        f.write_str("task_manager")
    }
}
